{-----------------------------------------------------------------------
                      BOXSHADE 2.7       (May 1994)

this program makes multiple-aligned output from either
 - PRETTY files from old LINEUP
 - MSF files from new PILEUP
 - CLUSTAL- .ALN  files
 - MALIGNED data files
 - ESEE files
various kinds of shading can be applied to identical/similar residues
Output formats supported are
 - ANSI for display on terminal with ANSI.SYS loaded (PC version)
 - VT100 for display on VT100 compatible terminals
 - CRT for display on the main screen ( * PC-version only! * )
 - POSTSCRIPT (TM)
 - Encapsulated POSTSCRIPT (for export to graphics programs)
 - HPGL for output on plotters or export to graphics programs
 - RTF (Rich text format) for export to word processing programs
 - ReGIS graphics for disply on terminals with ReGIS capabilities (DEC)
 - ReGIS file for prionting after conversion with RETOS
 - LJ250 coloir printer format (DEC)

This program might still contain bugs and is not particularly
user-friendly. It is completely public-domain and may be
passed around and modified without any notice to the author.

the authors address is:

  Kay Hofmann
  Bioinformatics Group
  ISREC
  Ch-1066 Epalinges s/Lausanne
  Switzerland

for any comments write an E-mail to
  khofmann@isrec-sun1.unil.ch

MODIFICATION HISTORY

2.7   -unified source code for VAX/VMS, AXP/OSF1 and MSDOS (Turbo Pascal)
      -all features of previous 2.6DOS version available for OSF1 and VMS
      -reads parameter and alignment files in both DOS and UNIX format
      -writes output files in native format of the machine. UNIX or DOS
       style text files can be forced by the qualifiers /unix or /dos
      -added /check qualifier for listing of command line parameters
      -added the option to hide the master sequence (see documentation)
      -finally dropped UIS support
      -minor bug fixes

2.6DOS   -added RTF output

2.5DOS   -added HPGL output
         -added consensus formation and output
         -added the /NUMDEF qualifier for default numbering
         -added the /SPLIT qualifier for splitting output to one page per file.
         -added the /CONS and
                    /SYMBCONS qualifiers for output of consensus sequence

2.4DOS   -initial MSDOS version
         -ported from VMS version 2.4
         -added CRT support 

2.4a  -fixed a bug (terminal size problems)

2.4   -modified POSTSCRIPT/EPS output, allows scaling of fonts.
       (.PAR file structure modified, BOX.PSP now obsolete)
      -support of vertical POSTSCRIPT output
      -support of output on VT-series terminals
      -support of input from CLUSTAL V
      -slight reordering of interactive input
      -minor bug in command line handling fixed
      -minor bug in EPS-bounding box fixed

2.3   -default parameter management
      -option for shading according to similarity to a single sequence
       instead of a consensus sequence
      -support of GCG V7 MSF-format

2.2   -support of sequence numbering in output
      -support of MALIGNED data files

2.1   -output code completely rewritten for allowing easier modification
      -support of ENCAPSULATED POSTSCRIPT output
      -more compact POSTSCRIPT output files
      -multipage output for all devices
      -modifications in PRETTY and CLUSTAL reading routines
       for coping with slightly altered formats

2.0   -added POSTSCRIPT and ReGIS output routines
      -added a version that compiles without VWS (BOX_NO_UIS)
      -some minor bugfixes
      -creation of a .COM file for setting logicals

1.0   -initial version
      -CLUSTAL and PRETTY input
      -UIS/VWS and LJ250 sixel output

----------------------------------------------------------------------- }

{$ifdef osf1}
  [INHERIT ('dep_dec.pen','bx_types.pen','bx_read.pen','bx_dev.pen')]
  program box(input,output);
{$endif}

{$ifdef vms}
  [INHERIT ('dep_vms.pen','bx_types.pen','bx_read.pen','bx_dev.pen')]
  program box(input,output);
{$endif}

{$ifdef Borland_Pascal}
  program box(input,output);
  uses dep_tp,bx_types,bx_read,bx_dev;
{$endif}



{--------------------------------------------------------------------------}
{   user query section section                                             }
{--------------------------------------------------------------------------}

function explain_cl : string255;
var ncl : string255;
begin
  writeln('Allowed command line parameters are:');
  writeln('/check          show this text');
  writeln('/def            use defaults, no unnecessary questions');
  writeln('/numdef         use default numbering');
  writeln('/dna            assume DNA sequences, use box_dna.par');
  writeln('/split          create separate files for multiple pages');
  writeln('/in=xxxxx       xxxxx is input file name');
  writeln('/out=xxxxx      xxxxx is output file name');
  writeln('/par=xxxxx      xxxxx is parameter file name');
  writeln('/dev=x          x is output device class (see documentation)');
  writeln('/type=x         x is input file format   (see documentation)');
  writeln('/cons           create consensus line');
  writeln('/symbcons=xyz   xyz are consensus symbols');
  writeln('/symbcons="xyz"    if the one above does not work, try this one');
  writeln('/unix           output files lines are terminated with LF only');
  writeln(' ');
  writeln('On unix systems, use the dash (-) as parameter delimiter ');
  writeln(' ');
  write('add to command line: ');
  readln(ncl);
  explain_cl:=ncl;
end;


function get_cl_filename(cl : cllinetype; tag : cltagtype) : string80;
var p1,p2,p3 : integer;
    cl2      : cllinetype;
begin
  p1:=indx(cl,tag);
  cl2:=substr(cl,p1+length(tag),length(cl)-(p1+length(tag))+1);
  p2:=indx(cl2,'-'); if p2=0 then p2:=length(cl2)+1;
  p3:=indx(cl2,' '); if p3=0 then p3:=length(cl2)+1;
  get_cl_filename:=substr(cl2,1,min(p2,p3)-1);
end;


procedure process_command_line;
var cl   : cllinetype;
    cs   : char;
    incr : integer;
begin
  cl:=get_command_line;
  if indx(cl,c_sep+'check')>0 then cl:=cl+' '+explain_cl;
  if indx(cl,c_sep+'dna')>0 then dnaflag:=true else dnaflag:=false;
  if indx(cl,c_sep+'def')>0 then interactflag:=false else interactflag:=true;
  if indx(cl,c_sep+'in=')>0 then begin
    clinflag:=true;
    inname:=get_cl_filename(cl,c_sep+'in=');
  end else clinflag:=false;
  if indx(cl,c_sep+'out=')>0 then begin
    cloutflag:=true;
    outname:=get_cl_filename(cl,c_sep+'out=');
  end else cloutflag:=false;
  if indx(cl,c_sep+'par=')>0 then begin
    clparflag:=true;
    parname:=get_cl_filename(cl,c_sep+'par=');
  end else clparflag:=false;
  if indx(cl,c_sep+'type=')>0 then begin
    cltypeflag:=true;
    inputmode:=cl[indx(cl,c_sep+'type=')+6];
  end else cltypeflag:=false;
  if indx(cl,c_sep+'dev=')>0 then begin
    cldevflag:=true;
    outputmode:=cl[indx(cl,c_sep+'dev=')+5];
  end else cldevflag:=false;
  if indx(cl,c_sep+'cons')>0 then consflag:=true else consflag:=false;
  if indx(cl,c_sep+'symbcons=')>0 then begin
    clsymbconsflag:=true;
    incr:=0;
    if (cl[indx(cl,c_sep+'symbcons=')+10]=chr(34)) or
       (cl[indx(cl,c_sep+'symbcons=')+10]=chr(39)) then incr:=1;
    symbcons[1]:=cl[indx(cl,c_sep+'symbcons=')+10+incr];
    symbcons[2]:=cl[indx(cl,c_sep+'symbcons=')+11+incr];
    symbcons[3]:=cl[indx(cl,c_sep+'symbcons=')+12+incr];
  end else clsymbconsflag:=false;
  if indx(cl,c_sep+'split')>0 then splitflag:=true else splitflag:=false;
  unixflag:=unixmode;
  if indx(cl,c_sep+'unix')>0 then unixflag:=true;
  if indx(cl,c_sep+'dos')>0 then unixflag:=false;
  if indx(cl,c_sep+'numdef')>0 then numdefflag:=true else numdefflag:=false;
end;


procedure ask;
var i,j             : integer;
    ok              : boolean;
    instring        : string50;

begin
  if not clparflag then begin
    if dnaflag then parname:='box_dna.par'
               else parname:='box_pep.par';
  end;
  if not fexist(parname) then begin
    if dnaflag then parname:=get_logical('BOXDIR')+d_sep+'box_dna.par'
               else parname:=get_logical('BOXDIR')+d_sep+'box_pep.par';
  end;
  if not fexist(parname) then begin
    ok:=false;
    repeat
      write('Default-parameter file ',parname,
            ' does not exist, enter filename: ');
      readln(parname);
      ok:=fexist(parname);
      if not ok then writeln(chr(7),'file does not exist ');
    until ok;
  end;
  fopen(parfile,parname,'r');
  repeat 
    readln(parfile,line) ;
  until indx(line,':GENERAL')=1;
  readln(parfile,line);
  if not clinflag then inputmode:=line[1];
  readln(parfile,line);
  if not cldevflag then outputmode:=line[1];
  readln(parfile,line); if line[1] in ['y','Y'] then seqconsflag:=true
                                                else seqconsflag:=false;
  hideflag :=false;
  readln(parfile,outlen);
  readln(parfile,line); if line[1] in ['y','Y'] then seqnameflag:=true
                                                else seqnameflag:=false;
  readln(parfile,line); if line[1] in ['y','Y'] then seqnumflag:=true
                                                else seqnumflag:=false;
  readln(parfile,interlines);
  readln(parfile,line); if line[1] in ['y','Y'] then simflag:=true
                                                else simflag:=false;
  readln(parfile,line); if line[1] in ['y','Y'] then globalflag:=true
                                                else globalflag:=false;

  readln(parfile,line);
  if not consflag then if line[1] in ['y','Y'] then consflag:=true
                                               else consflag:=false;

  readln(parfile,line);
  if not clsymbconsflag then begin
    symbcons[1]:=line[1];
    symbcons[2]:=line[2];
    symbcons[3]:=line[3];
  end;

  readln(parfile,line);
  if not splitflag then if line[1] in ['y','Y'] then splitflag:=true
                                                else splitflag:=false;

  readln(parfile,line);
  if not numdefflag then if line[1] in ['y','Y'] then numdefflag:=true
                                                 else numdefflag:=false;


  for i:=1 to max_no_seq do for j:=1 to max_no_res do seq[i,j]:=' ';
  writeln(' ');
  writeln('This program makes multiple-aligned output from either');
  writeln('PILEUP-MSF, CLUSTAL-ALN, MALIGNED-data and ESEE-save files');
  writeln('Various kinds of shading can be applied to identical/similar residues');
  writeln('Output is written to screen or to a file in the following formats:');
  writeln('ANSI/VT100, PS/EPS, RTF, HPGL, ReGIS, LJ250-printer');
  writeln(' ');
  {*** ask for infile ***}
  instring:='';
  ok:=false;
  repeat
    if not clinflag then begin
      write('name of aligned input-file : ');
      readln(inname);
      writeln(' ');
    end;
    ok:=fexist(inname);
    if not ok then begin
      writeln(chr(7),'aligned input file does not exist ');
      clinflag:=false;
    end;
  until ok;
  {*** ask for infile type ***}
  if not cltypeflag then begin
    if (indx(inname,'.pre')>0) or (indx(inname,'.PRE')>0)
    or (indx(inname,'.msf')>0) or (indx(inname,'.MSF')>0)
      then inputmode:='1' else
    if (indx(inname,'.aln')>0) or (indx(inname,'.ALN')>0)
      then inputmode:='2' else
    if (indx(inname,'.mal')>0) or (indx(inname,'.MAL')>0)
      then inputmode:='3' else
    if (indx(inname,'.ese')>0) or (indx(inname,'.ESE')>0)
      then inputmode:='4';
    if (indx(inname,'.phy')>0) or (indx(inname,'.PHY')>0)
      then inputmode:='5';
  end;
  if interactflag and (not cltypeflag) then repeat
    writeln('Do you want to process (1) Lineup-PRETTY/Pileup-MSF file');
    writeln('                       (2) CLUSTAL .ALN file');
    writeln('                       (3) MALIGNED data file');
    writeln('                       (4) ESEE save file');
    write('                       (5) PHYLIP file   (* ',inputmode,' *) : ');
    readln(instring);
    if instring='' then instring:=inputmode;
    if indx('12345',instring)=0 then begin
      writeln(' ');
      writeln(chr(7),'---> Please choose a supported type');
      writeln(' ');
    end;
  until indx('12345',instring)>0;
  writeln(' ');
  if instring <>'' then inputmode:=instring[1];

  instring:='';
  if not cldevflag then repeat

{$ifdef Borland_Pascal}
    writeln('Output suitable for (',   oPS :2,') POSTSCRIPT');
    writeln('                    (',  oEPS :2,') encapsulated POSTSCRIPT');
    writeln('                    (', oHPGL :2,') HPGL');
    writeln('                    (',  oRTF :2,') RTF (Rich Text Format)');
    writeln('                    (',  oCRT :2,') PC-screen     (PCs only!)');
    writeln('                    (', oANSI :2,') ANSI-screen   (PC-version)');
    writeln('                    (',   oVT :2,') VT100-screen  (DEC-version)');
    writeln('                    (',oREGISt:2,') ReGIS-screen  (25 lines each');
    writeln('                    (',oREGISp:2,') ReGIS-file    (without breaks)');
    writeln('                    (',oLJ250 :2,') LJ250-printer file');
    write('                      current:   (* ',outputmode,' *) : ');
{$endif}
{$ifdef dec}
    writeln('Output suitable for (',    oPS:2,') POSTSCRIPT');
    writeln('                    (',   oEPS:2,') encapsulated POSTSCRIPT');
    writeln('                    (',  oHPGL:2,') HPGL');
    writeln('                    (',   oRTF:2,') RTF (Rich Text Format)');
    writeln('                    (',  oANSI:2,') ANSI-screen   (PC-version)');
    writeln('                    (',    oVT:2,') VT100-screen  (DEC-version)');
    writeln('                    (',oREGISt:2,') ReGIS-screen  (25 lines each');
    writeln('                    (',oREGISp:2,') ReGIS-file    (without breaks)');
    writeln('                    (',oLJ250 :2,') LJ250-printer file');
    write('                      current:  (* ',outputmode,' *) : ');
{$endif}

    readln(instring);
    if instring='' then instring:=outputmode;
    if indx(allowed_devices,instring)=0 then begin
      writeln(' ');
      writeln(chr(7),'---> Please choose a supported type');
      writeln(' ');
    end
  until indx(allowed_devices,instring)>0;
  if instring<>'' then outputmode:=instring[1];
  writeln(' ');

  instring:='';
  if interactflag then  repeat
    writeln('-------------------------------------------------------------------');
    if seqconsflag then write('similarity to a single sequence? (* y *) : ')
                   else write('similarity to a single sequence? (* n *) : ');
    readln(instring);
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then seqconsflag:=true
                                                  else seqconsflag:=false;

  instring:='';
  if interactflag and seqconsflag then  repeat
    writeln('-------------------------------------------------------------------');
    if hideflag then write('hide this sequence? (* y *) : ')
                else write('hide this sequence? (* n *) : ');
    readln(instring);
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then hideflag:=true
                                                  else hideflag:=false;
  instring:='';
  if interactflag then  repeat
    writeln('-------------------------------------------------------------------');
    if consflag then write('display consensus line ? (* y *) : ')
                else write('display consensus line ? (* n *) : ');
    readln(instring);
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then consflag:=true
                                                  else consflag:=false;
  instring:='';
  if interactflag and consflag and (not clsymbconsflag) then  begin
    writeln(' ');
    writeln('Enter now a string of 3 symbols to be used for consensus display');
    writeln('          representing normal/all-similar/all-identical residues');
    writeln('          see documentation for symbol definition, examples are:');
    writeln('          " .*" or "-LU"    (ommit quotes but use blanks)       ');
    write('SYMBOL : '); readln(instring);
    if instring<>'' then begin
      while length(instring)<>3 do begin
        writeln(chr(7),'please enter string of THREE (3) symbols !');
        write('SYMBOL : '); readln(instring);
      end;
      symbcons[1]:=instring[1];
      symbcons[2]:=instring[2];
      symbcons[3]:=instring[3];
    end;
  end;
  instring:='';
  if interactflag then  repeat
    writeln('-------------------------------------------------------------------');
    writeln(' ');
    write('How many sequence characters per line (* ',outlen:3,' *) : ');
    readln(instring);
    if instring<>'' then outlen:=str2int(instring);
    writeln(' ');
  until (outlen>10) and (outlen<max_outlen);

  instring:='';
  if interactflag then  repeat
    if seqnameflag then write('should sequence name be printed (* y *) : ')
                   else write('should sequence name be printed (* n *) : ');
    readln(instring);
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then seqnameflag:=true
                                                  else seqnameflag:=false;

  instring:='';
  if interactflag then  repeat
    if seqnumflag then write('should position numbers be printed (* y *) : ')
                  else write('should position numbers be printed (* n *) : ');
    readln(instring);
    writeln(' ');
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then seqnumflag:=true
                                                  else seqnumflag:=false;

  instring:='';
  if interactflag then  repeat
    write('How many lines between two sequence blocks (* ',interlines:2,' *) : ');
    readln(instring);
    if instring<>'' then interlines:=str2int(instring);
    writeln(' ');
  until (interlines>0) and (interlines<100);

  instring:='';
  if interactflag then  repeat
    if simflag then write('special label for similar residues (* y *) : ')
               else write('special label for similar residues (* n *) : ');
    readln(instring);
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then simflag:=true
                                                  else simflag:=false;
  if simflag then begin
    if dnaflag then cmpname:='box_dna.sim'
               else cmpname:='box_pep.sim';
    if not fexist(cmpname) then begin
      if dnaflag then cmpname:=get_logical('BOXDIR')+d_sep+'box_dna.sim'
                 else cmpname:=get_logical('BOXDIR')+d_sep+'box_pep.sim';
    end;
    if not fexist(cmpname) then begin
      ok:=false;
      repeat
        write('Similarity-file ',cmpname,' does not exist, enter filename: ');
        readln(cmpname);
        ok:=fexist(cmpname);
        if not ok then writeln(chr(7),'file does not exist ');
      until ok;
    end;
  end;

  instring:='';
  if interactflag then  repeat
    if globalflag then write('special label for identical residues in all sequences (* y *) : ')
               else write('special label for identical residues in all sequences (* n *) : ');
    readln(instring);
  until (indx('nNyY',instring)>0) or (instring='');
  if instring<>'' then if upcase(instring[1])='Y' then globalflag:=true
                                                  else globalflag:=false;

  for i:=0 to 3 do lc[i]:=false;

{$ifdef dec}
  case outputmode of
  oANSI     : ask_ansi;
  oPS       : ask_postscript;
  oEPS      : ask_postscript;
  oHPGL     : ask_hpgl;
  oRTF      : ask_rtf;
  oREGISt   : ask_regis;
  oREGISp   : ask_regis;
  oVT       : ask_vt;
  oLJ250    : ask_lj250
  end;
{$endif}

{$ifdef Borland_Pascal}
  case outputmode of
  oCRT      : ask_crt;
  oANSI     : ask_ansi;
  oPS       : ask_postscript;
  oEPS      : ask_postscript;
  oHPGL     : ask_hpgl;
  oRTF      : ask_rtf;
  oREGISt   : ask_regis;
  oREGISp   : ask_regis;
  oVT       : ask_vt;
  oLJ250    : ask_lj250
  end;
{$endif}

  close(parfile);
end;

procedure ask_numbers;
var instring : string10;
    inno     : integer;
    i        : integer;
begin
  writeln('-------------------------------------------------------');
  writeln(' ');
  writeln('You requested sequence numbering in the output.');
  writeln('Enter the number of the first sequence position');
  writeln('or confirm the suggestion of the program');

  for i:=1 to no_seq do begin
    write(seqname[i],' (* ',startno[i]:4,' *)  : ');
    readln(instring);
    if instring<>'' then begin
      inno:=str2int(instring);
      if depend_err<>'str2int' then startno[i]:=inno;
    end else startno[i]:=1;
  end;
  writeln(' ');
end;

procedure ask_seqcons;
var instring : string10;
    i        : integer;
begin
  writeln('-------------------------------------------------------');
  writeln(' ');
  writeln('You requested consensus formation to a single sequence.');
  writeln('Choose now the sequence to compare to the other ones.');
  for i:=1 to no_seq do begin
    writeln('(',i:2,') ',seqname[i]);
  end;
  repeat
    write('No. of sequence: (* 1 *) : ');
    readln(instring);
    if instring='' then consensnum:=1 else consensnum:=str2int(instring);
  until (consensnum>=1) and (consensnum<=no_seq);
  writeln(' ');
end;


{--------------------------------------------------------------------------}
{   read input files and comparison-table                                  }
{--------------------------------------------------------------------------}


procedure read_cmp;
var i,j     : integer;
    p1,p2   : integer;
    cmpfile : text;
begin
  if simflag then begin
    for i:=1 to 20 do for j:=1 to 20 do cmptable[i,j]:=false;
    for i:=1 to 20 do cmptable[i,i]:=true;
    fopen(cmpfile,cmpname,'r');
    repeat
      readln(cmpfile,line);
    until indx(line,'..')>0;
    repeat
      repeat readln(cmpfile,line) until length(line)>0;
      p1:=indx(aaset,line[1]);
      if p1>0 then for i:=3 to length(line) do begin
        p2:=indx(aaset,line[i]);
        if p2>0 then begin
          cmptable[p1,p2]:=true;
          cmptable[p2,p1]:=true;
        end;
      end;
    until eof(cmpfile);
  end;
end;


{--------------------------------------------------------------------------}
{  compare sequences and do shading                                        }
{--------------------------------------------------------------------------}

function sim(a,b : char) : boolean;
var p1,p2 : integer;
begin
  p1:=indx(aaset,a);
  p2:=indx(aaset,b);
  if (p1>0) and (p2>0) then
    sim:=cmptable[p1,p2]
  else sim:=false;
end;

procedure make_consensus;
var i,j,k    : integer;
    count    : array[1..max_no_seq] of integer;
    maxcount : integer;
begin
  consenslen:=0;
  for i:=1 to no_seq do begin
    while seq[i,seqlen[i]] in [' ','.','-'] do seqlen[i]:=seqlen[i]-1;
    if seqlen[i]>consenslen then consenslen:=seqlen[i];
  end;
  if not seqconsflag then begin
    for i:=1 to consenslen do begin
      for j:=1 to no_seq do begin
        count[j]:=0;
        for k:=1 to no_seq do begin
          if (seq[k,i]=seq[j,i]) and (indx(aaset,seq[j,i])>0)
            then count[j]:=count[j]+idpref;
          if sim(seq[k,i],seq[j,i]) and (indx(aaset,seq[j,i])>0)
            then count[j]:=count[j]+simpref;
        end;
      end;
      cons[i]:=seq[1,i];
      maxcount:=count[1];
      for j:=2 to no_seq do begin
        if count[j]>maxcount then begin
          cons[i]:=seq[j,i];
          maxcount:=count[j];
        end;
      end;
    end;
  end else begin
    for i:=1 to consenslen do begin
      cons[i]:=seq[consensnum,i]
    end;
  end;
end;


procedure make_colors;
var i,j              : integer;
    idcount,simcount : integer;
begin
  for i:=1 to consenslen do begin
    idcount:=0; simcount:=0;
    for j:=1 to no_seq do begin
      col[j,i]:=0;
      if seq[j,i]=cons[i] then idcount:=idcount+1 else
      if sim(seq[j,i],cons[i]) then simcount:=simcount+1;
    end;
    if (idcount=no_seq) and (indx(aaset,cons[i])>0) then begin
      for j:=1 to no_seq do col[j,i]:=3;
    end
    else if (idcount>1) and (indx(aaset,cons[i])>0) then begin
      for j:=1 to no_seq do begin
         if seq[j,i]=cons[i] then col[j,i]:=1
         else if sim(seq[j,i],cons[i]) then col[j,i]:=2;
      end;
    end
    else if (simcount>0) and (indx(aaset,cons[i])>0) then begin
      for j:=1 to no_seq do begin
         if sim(seq[j,i],cons[i]) then col[j,i]:=2;
      end;
    end;
  if consflag then begin
      if idcount=no_seq then begin
        conschar[i]:=symbcons[3];
        if upcase(conschar[i])='U' then conschar[i]:=upcase(cons[i])
        else if upcase(conschar[i])='L' then conschar[i]:=lowcase(cons[i])
        else if upcase(conschar[i])='B' then conschar[i]:=' ';
      end
      else if (idcount+simcount)=no_seq then begin
        conschar[i]:=symbcons[2];
        if upcase(conschar[i])='U' then conschar[i]:=upcase(cons[i])
        else if upcase(conschar[i])='L' then conschar[i]:=lowcase(cons[i])
        else if upcase(conschar[i])='B' then conschar[i]:=' ';
      end
      else begin
        conschar[i]:=symbcons[1];
        if upcase(conschar[i])='U' then conschar[i]:=upcase(cons[i])
        else if upcase(conschar[i])='L' then conschar[i]:=lowcase(cons[i])
        else if upcase(conschar[i])='B' then conschar[i]:=' ';
      end;
    end;
  end;
end;

procedure make_lowcase;
var i,j : integer;
begin
  for i:=1 to no_seq do for j:=1 to seqlen[i] do begin
  case col[i,j] of
     0 : if lc[0] then seq[i,j]:=lowcase(seq[i,j]);
     1 : if lc[1] then seq[i,j]:=lowcase(seq[i,j]);
     2 : if lc[2] then seq[i,j]:=lowcase(seq[i,j]);
     3 : if lc[3] then seq[i,j]:=lowcase(seq[i,j]);
    end;
  end;
end;

procedure prepare_names;
var i,j,p : integer;
begin
  seqname_outlen:=1;
  for i:=1 to no_seq do begin
    while seqname[i,length(seqname[i])]=' ' do
      seqname[i]:=substr(seqname[i],1,length(seqname[i])-1);
    if length(seqname[i])>seqname_outlen 
      then seqname_outlen:=length(seqname[i]);
  end;
  for i:=1 to no_seq do begin
    p:=seqname_outlen-length(seqname[i]);
    if p>0 then for j:=1 to p do seqname[i]:=seqname[i]+' ';
  end;
end;

procedure prepare_numbers;
var count,bn,i,j,k : integer;
begin
  for i:=1 to no_seq do begin
    for j:=1 to max_no_block do begin
      prenum[i,j]:='';
      for k:=1 to seqnumlen do prenum[i,j]:=prenum[i,j]+' ';
    end;
    prenum[i,1]:=int2str(startno[i],seqnumlen);
    count:=startno[i]-1;
    bn:=1;
    for j:=1 to seqlen[i] do begin
      if (seq[i,j] in ['A'..'Z']) or (consflag and (i=no_seq))
        then count:=count+1;
      if (j mod outlen)=0 then begin
        bn:=bn+1;
        if (count+1)<(seqlen[i]+startno[i])
          then prenum[i,bn]:=int2str(count+1,seqnumlen); 
      end;
    end;
  end;
end;

{--------------------------------------------------------------------------}
{   graphics section                                                       }
{--------------------------------------------------------------------------}

procedure graphics_init(var xpos,ypos : real);
const dummychar = ' ';
      dummycolor = 0;
begin
{$ifdef Borland_Pascal}
  if outputmode=oCRT then 
    crtout(init,xpos,ypos,dummychar,dummycolor)
  else 
{$endif}
  if outputmode=oANSI then 
    ansi(init,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oPS then 
    postscript(init,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oEPS then 
    eps(init,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oHPGL then 
    hpgl(init,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oLJ250 then 
    lj250(init,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oVT then 
    vt(init,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oREGISt then 
    regis(init,xpos,ypos,dummychar,dummycolor,rTERM)
  else 
  if outputmode=oREGISp then 
    regis(init,xpos,ypos,dummychar,dummycolor,rPAGE)
  else 
  if outputmode=oRTF then 
    rtf(init,xpos,ypos,dummychar,dummycolor);
  
  blocks_per_page:= trunc((dev_maxy-dev_miny)/(dev_ysize*(no_seq+interlines)));
  act_page:=1;
end;

procedure graphics_setcolor(colno : integer);
const dummychar = ' ';
var   dummypos : real;
begin
  if colno<>act_color then begin
    dummypos:=0;
{$ifdef Borland_Pascal}
    if outputmode=oCRT then 
      crtout(setcol,dummypos,dummypos,dummychar,colno)
    else 
{$endif}
    if outputmode=oANSI then 
      ansi(setcol,dummypos,dummypos,dummychar,colno)
    else 
    if outputmode=oPS then 
      postscript(setcol,dummypos,dummypos,dummychar,colno)
    else 
    if outputmode=oEPS then 
      eps(setcol,dummypos,dummypos,dummychar,colno)
    else 
    if outputmode=oHPGL then 
      hpgl(setcol,dummypos,dummypos,dummychar,colno)
    else 
    if outputmode=oVT then 
      vt(setcol,dummypos,dummypos,dummychar,colno)
    else 
    if outputmode=oREGISt then 
      regis(setcol,dummypos,dummypos,dummychar,colno,rTERM)
    else 
    if outputmode=oREGISp then 
      regis(setcol,dummypos,dummypos,dummychar,colno,rPAGE)
    else 
    if outputmode=oLJ250 then 
      lj250(setcol,dummypos,dummypos,dummychar,colno)
    else 
    if outputmode=oRTF then 
      rtf(setcol,dummypos,dummypos,dummychar,colno);
    act_color:=colno;
  end;
end;

procedure graphics_charout(c : char; var xpos,ypos : real);
const dummycolor = 0;
begin
{$ifdef Borland_Pascal}
  if outputmode=oCRT then 
    crtout(charout,xpos,ypos,c,dummycolor)
  else 
{$endif}
  if outputmode=oANSI then 
    ansi(charout,xpos,ypos,c,dummycolor)
  else 
  if outputmode=oPS then 
    postscript(charout,xpos,ypos,c,dummycolor)
  else 
  if outputmode=oEPS then 
    eps(charout,xpos,ypos,c,dummycolor)
  else 
  if outputmode=oHPGL then 
    hpgl(charout,xpos,ypos,c,dummycolor)
  else 
  if outputmode=oVT then 
    vt(charout,xpos,ypos,c,dummycolor)
  else 
  if outputmode=oREGISp then 
    regis(charout,xpos,ypos,c,dummycolor,rPAGE)
  else 
  if outputmode=oREGISt then 
    regis(charout,xpos,ypos,c,dummycolor,rTERM)
  else 
  if outputmode=oLJ250 then 
    lj250(charout,xpos,ypos,c,dummycolor)
  else 
  if outputmode=oRTF then 
    rtf(charout,xpos,ypos,c,dummycolor)
end;

procedure graphics_newline(var xpos,ypos : real);
const dummychar   = ' ';
      dummycolor  = 0;
begin
{$ifdef Borland_Pascal}
  if outputmode=oCRT then 
    crtout(newline,xpos,ypos,dummychar,dummycolor)
  else 
{$endif}
  if outputmode=oANSI then 
    ansi(newline,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oPS then 
    postscript(newline,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oEPS then 
    eps(newline,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oHPGL then 
    hpgl(newline,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oVT then 
    vt(newline,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oREGISp then 
    regis(newline,xpos,ypos,dummychar,dummycolor,rPAGE)
  else 
  if outputmode=oREGISt then 
    regis(newline,xpos,ypos,dummychar,dummycolor,rTERM)
  else 
  if outputmode=oLJ250 then 
    lj250(newline,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oRTF 
    then rtf(newline,xpos,ypos,dummychar,dummycolor);
end;

procedure graphics_newpage(var xpos,ypos : real);
const dummychar   = ' ';
      dummycolor  = 0;
begin
{$ifdef Borland_Pascal}
  if outputmode=oCRT then 
    crtout(newpage,xpos,ypos,dummychar,dummycolor)
  else 
{$endif}
  if outputmode=oANSI then 
    ansi(newpage,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oPS then 
    postscript(newpage,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oEPS then 
    eps(newpage,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oHPGL then 
    hpgl(newpage,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oVT then 
    vt(newpage,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oREGISp then 
    regis(newpage,xpos,ypos,dummychar,dummycolor,rPAGE)
  else 
  if outputmode=oREGISt then 
    regis(newpage,xpos,ypos,dummychar,dummycolor,rTERM)
  else 
  if outputmode=oLJ250 then 
    lj250(newpage,xpos,ypos,dummychar,dummycolor)
  else 
  if outputmode=oRTF then 
    rtf(newpage,xpos,ypos,dummychar,dummycolor);
  act_page:=act_page+1;
end;

procedure graphics_exit;
const dummychar  = ' ';
      dummycolor = 0;
var   dummypos   : real;
begin
  dummypos:=0;
{$ifdef Borland_Pascal}
    if outputmode=oCRT then 
      crtout(exit,dummypos,dummypos,dummychar,dummycolor)
    else 
{$endif}
    if outputmode=oANSI then 
      ansi(exit,dummypos,dummypos,dummychar,dummycolor)
    else if outputmode=oPS then 
      postscript(exit,dummypos,dummypos,dummychar,dummycolor)
    else 
    if outputmode=oEPS then 
      eps(exit,dummypos,dummypos,dummychar,dummycolor)
    else 
    if outputmode=oHPGL then 
      hpgl(exit,dummypos,dummypos,dummychar,dummycolor)
    else 
    if outputmode=oVT then 
      vt(exit,dummypos,dummypos,dummychar,dummycolor)
    else 
    if outputmode=oREGISp then 
      regis(exit,dummypos,dummypos,dummychar,dummycolor,rPAGE)
    else 
    if outputmode=oREGISt then 
      regis(exit,dummypos,dummypos,dummychar,dummycolor,rTERM)
    else 
    if outputmode=oLJ250 then 
      lj250(exit,dummypos,dummypos,dummychar,dummycolor)
    else 
    if outputmode=oRTF then 
      rtf(exit,dummypos,dummypos,dummychar,dummycolor)
end;


procedure graphics_out;
var
    xpos,ypos  : real;    {current position of text cursor}
    i,j,k      : integer;
    pagesymbol : char;
begin
  pagesymbol:='1';
  act_color:=-1;
  no_blocks:=((consenslen-1) div outlen) + 1;
  if consflag then begin
    no_seq:=no_seq+1;
    seqlen[no_seq]:=consenslen;
    seqname[no_seq]:='consensus';
    startno[no_seq]:=1;
    for i:=1 to consenslen do begin
      seq[no_seq,i]:=conschar[i];
      col[no_seq,i]:=0;
    end;
  end;
  if seqconsflag and hideflag then begin
    if consensnum<no_seq then for i:=consensnum to no_seq-1 do begin
      seqlen[i]:=seqlen[i+1];
      seqname[i]:=seqname[i+1];
      startno[i]:=startno[i+1];
      seq[i]:=seq[i+1];
      col[i]:=col[i+1];
    end;
    no_seq:=no_seq-1;
  end;

  if seqnameflag then prepare_names;
  if seqnumflag then prepare_numbers;

  graphics_init(xpos,ypos);
  for i:=0 to (no_blocks-1) do begin
    for j:=1 to no_seq do begin
      if seqnameflag then begin
        graphics_setcolor(0);
        for k:=1 to seqname_outlen do begin
          graphics_charout(seqname[j,k],xpos,ypos);
        end;
        graphics_charout(' ',xpos,ypos);
      end;
      if seqnumflag then begin
        graphics_setcolor(0);
        for k:=1 to length(prenum[j,i+1]) do begin
          graphics_charout(prenum[j,i+1,k],xpos,ypos);
        end;
        graphics_charout(' ',xpos,ypos);
      end;
      for k:=1 to outlen do if (i*outlen+k)<=consenslen then begin
        graphics_setcolor(col[j,i*outlen+k]);
        graphics_charout(seq[j,i*outlen+k],xpos,ypos);
      end;
      graphics_newline(xpos,ypos);
    end;
    if ((i+1) mod blocks_per_page) <> 0 then
      for j:=1 to interlines do graphics_newline(xpos,ypos);
    if (((i+1) mod blocks_per_page)=0) and (i<(no_blocks-1))
    then begin
      if splitflag then begin
        graphics_exit;
        pagesymbol:=chr(ord(pagesymbol)+1);
        outname[indx(outname,'.')-1]:=pagesymbol;
        graphics_init(xpos,ypos);
      end else graphics_newpage(xpos,ypos);
    end;
  end;
  graphics_exit;
end;

{--------------------------------------------------------------------------}
{   main program                                                           }
{--------------------------------------------------------------------------}

begin
  writeln(cmpname);
  process_command_line;
  ask;
  case inputmode of
    '1': read_file_pretty;
    '2': read_file_clustal;
    '3': read_file_maligned;
    '4': read_file_esee;
    '5': read_file_phylip;
  end;
  if seqnumflag and not numdefflag then ask_numbers;
  if seqconsflag then ask_seqcons;
  read_cmp;
  make_consensus;
  make_colors;
  make_lowcase;
  unixmode:=unixflag;
  graphics_out;
end.
